/**
 * \file abstractmatrix-private.hpp
 *
 * \authors Rowan Zellers
 *
 * \brief Private implementation file for the AbstractMatrix class.
 */

#ifndef ABSTRACTMATRIX_PRIVATE_HPP_INCLUDED
#define ABSTRACTMATRIX_PRIVATE_HPP_INCLUDED 1

#include <algorithm>
#include <cassert>
#include <string>
#include <vector>
#include <deque>
#include <iostream>
#include <fstream>
#include <sstream>

AbstractMatrix::AbstractMatrix(size_t type)
  : type_(type)
{
    // nothing else needed
}

size_t AbstractMatrix::type() const
{
    return type_;
}

bool AbstractMatrix::save(const std::string& filename) const
{
    // Check that we can write to the file, and get the output
    // specifications (like number of rows/columns).
    std::ofstream myfile;
    myfile.open(filename.c_str());
    if (!myfile.is_open()) {
        return false;
    }
    size_t numrows = rows();
    size_t numcols = columns();

    // The first line will announce the matrix's type.
    myfile << "Type: " << type_ << std::endl;

    // If numcols = 0, we have a background matrix. So we simply need
    // to output ID's, then the value of the matrix in that cell.
    if (numcols == 0) {
        for (size_t i = 0; i < numrows; ++i) {
            myfile << idToString(i) << ":\t" << getCell(i) << std::endl;
        }
        return true;
    }

    // We are now guaranteed to be printing out either a weightmatrix
    // or a freqmatrix, so print the rows and columns accordingly.
    for (size_t i = 0; i < numrows; ++i) {
        myfile << idToString(i) << ":";
        for (size_t j = 0; j < numcols; ++j) {
            myfile << "\t" << getCell(i,j);
        }
        myfile << std::endl;
    }
    myfile.close();
    return true;
}

std::string AbstractMatrix::strType() const
{
    // Get the positions of the relative nucleotides by
    // decomposing the type. We can find also the length of
    // the block of nucleotides this way. Consider the PATSER
    // case where the only nucleotide that is relevant is at position
    // 0 - then, the length of the block is 1. So, we add a 1 to the
    // maximal element in the type decomposition.
    std::vector<size_t> pos = decomposeType();
    size_t length = *std::max_element(pos.begin(), pos.end()) + 1;
    std::string rep("");

    // If a nucleotide position is relevant, insert "m", else
    // insert "k".
    for (size_t i = 0; i < length; ++i) {
        if (std::find(pos.begin(), pos.end(), i) == pos.end()) {
            rep.push_back('k');
        } else {
            rep.push_back('m');
        }
    }
    return std::string(rep.rbegin(), rep.rend());
}


std::vector<size_t> AbstractMatrix::decomposeType() const
{
   // Convert "type" into its binary representation.
   size_t i = type_;
   std::vector<size_t> binaryrep;
   size_t remainder;
   while (i > 1) {
       remainder = i % 2;
       binaryrep.push_back(remainder);
       i = i/2;
   }
   binaryrep.push_back(i);

   // If we see a 1 in the binary representation, then that's a nucleotide
   // that's relevant, so add it to the decomposed type.
   std::vector<size_t> decomposedType;
   decomposedType.push_back(0); // The decomposed type should ALWAYS have 0.
   for (size_t j = 0; j < binaryrep.size(); ++j) {
       if (binaryrep[j] == 1) {
           decomposedType.push_back(j+1);
       }
   }

    return decomposedType;
}

size_t AbstractMatrix::rows() const
{
    // The number of rows is 4^m where m is the number of relevant
    // nucleotides. The size of the decomposed type is m.
    std::vector<size_t> positions = decomposeType();
    size_t accumulator = 1;
    for (size_t i = 0; i < positions.size(); ++i) {
         accumulator *= 4;
    }
     return accumulator;
}

size_t AbstractMatrix::nucStringToID(const std::string& nucleotides)
{
    // Sanity check that there aren't any N's. I could potentially search
    // for anything that's not N, C, G, or T, but there should be
    // no need.
    if (nucleotides.find('N') != std::string::npos) {
        std::cerr << "There's an N " << std::endl;
        std::cerr << nucleotides << std::endl;
        return 0;
    }

    // Convert from base 4 to base 10.
    size_t ID = 0;
    std::string reversedString = std::string(nucleotides.rbegin(), 
        nucleotides.rend());
    size_t multiplier = 1;
    for (size_t i = 0; i < reversedString.length(); ++i) {
        if(reversedString[i] == 'C' || reversedString[i] == 'c' ) {
        ID += multiplier;
        } else if (reversedString[i] == 'G' || reversedString[i] == 'g') {
            ID += 2*multiplier;
        } else if (reversedString[i] == 'T' || reversedString[i] == 't') {
            ID += 3*multiplier;
        } else if (reversedString[i] != 'A' && reversedString[i] != 'a') {
            std::cerr << "ERROR with string " << nucleotides << std::endl;
        }
        multiplier *= 4;
    }
    return ID;
}

std::string AbstractMatrix::idToString(const size_t id) const
{
    // Convert from base 10 to base 4.
    size_t i = id;
    std::vector<size_t> baseFourID;
    size_t remainder;
    while (i > 3) {
        remainder = i % 4;
        baseFourID.push_back(remainder);
        i = i/4;
    }

    // Make a deque and fill it from the vector. This is because
    // we'll be popping from the front of it.
    baseFourID.push_back(i);
    std::deque<size_t> relevantNucleotides;
    std::string actualNucleotides("");
    std::vector<size_t> relevantNucVector = decomposeType();
    for (size_t i = 0; i < relevantNucVector.size(); ++i) {
        relevantNucleotides.push_back(relevantNucVector[i]);
    }

    size_t nucCount = 0;
    size_t strCount = 0;

    // Now, go through the deque and generate a DNA string from
    // it. Note that if the base 4 ID becomes empty, then we push
    // back "A"'s, due to how we performed the base 4 conversion.
    while (!relevantNucleotides.empty()) {
        if (nucCount == relevantNucleotides.front()) {
            if (baseFourID.size() <= strCount) {
              actualNucleotides.push_back('A');
            } else {
            if (baseFourID[strCount] == 0) {
                actualNucleotides.push_back('A');
                }
            if (baseFourID[strCount] == 1) {
                actualNucleotides.push_back('C');
                }
            if (baseFourID[strCount] == 2) {
                actualNucleotides.push_back('G');
            }
            if (baseFourID[strCount] == 3) {
                actualNucleotides.push_back('T');
            }
          }
          relevantNucleotides.pop_front();
          ++strCount;
      } else {
          actualNucleotides.push_back('N');
      }
       ++nucCount;
   }
   std::reverse(actualNucleotides.begin(), actualNucleotides.end());
   return actualNucleotides;
}

std::string AbstractMatrix::baseTenToNuc(const size_t id) const
{
    // Convert from base 10 to base 4.
    size_t i = id;
    std::vector<size_t> baseFourID;
    size_t remainder;
    while (i > 3) {
        remainder = i % 4;
        baseFourID.push_back(remainder);
        i = i/4;
    }
    baseFourID.push_back(i);

    // Get the window length.
    std::vector<size_t> positions = decomposeType();
    size_t startpos = *std::max_element(positions.begin(), positions.end());
    size_t wlength = columns() + startpos;
    std::string actualNucleotides("");

    // For each digit in the base 4 representation, push back the corresponding
    // nucleotide. If we run out of digits in the base 4 representation push
    // back A's, as A represents 0.
    for (size_t i = 0; i < wlength; ++i) {
        if (i >= baseFourID.size()) {
            actualNucleotides.push_back('A');
        } else {
            if (baseFourID[i] == 0) {
                actualNucleotides.push_back('A');
            }
            if (baseFourID[i] == 1) {
                actualNucleotides.push_back('C');
            }
            if (baseFourID[i] == 2) {
                actualNucleotides.push_back('G');
            }
            if (baseFourID[i] == 3) {
                actualNucleotides.push_back('T');
            }
        }
    }
    // Since we pushed back the largest digits most recently, reverse the
    // string.
    std::reverse(actualNucleotides.begin(), actualNucleotides.end());
    return actualNucleotides;
}

size_t AbstractMatrix::deref(const std::string& nucleotides, const size_t location) const
{
    std::vector<size_t> relevantNucleotides = decomposeType();
    std::string actualNucleotides("");

    // Push back each relevant nucleotide into the string, then generate
    // an ID from it.
    for (size_t i = 0; i < relevantNucleotides.size(); ++i) {
        actualNucleotides.push_back(
          nucleotides[location - relevantNucleotides[i]]);
    }
    std::reverse(actualNucleotides.begin(), actualNucleotides.end() );
    return nucStringToID(actualNucleotides);
}


#endif
