/**
 * \file abstractmatrix.hpp
 *
 * \author Rowan Zellers
 *
 * \brief An abstract class representing a matrix.
 */

#ifndef ABSTRACTMATRIX_HPP_INCLUDED
#define ABSTRACTMATRIX_HPP_INCLUDED 1

#include <iostream>
#include <vector>

double SMOOTHING_CONSTANT = 0.01;

size_t VISIBLE_DIGITS = 6;

/**
 * \class AbstractMatrix
 * \brief An abstract matrix is an abstract class representing a matrix. It
 *    has a type corresponding to which nucleotides it considers.
 * \remarks WeightMatrices, FrequencyMatrices, and BackgroundMatrices
 *    all descend from this class. This is deemed acceptable because it means
 *    I don't have to duplicate code.
 */
class AbstractMatrix {
public:
    /**
     * \brief Constructs an abstractmatrix based on its type.
     */
    AbstractMatrix(const size_t type=0);

    size_t rows() const; ///< Returns the number of rows.

    size_t type() const; ///< returns the type (as an integer).

    virtual size_t columns() const = 0; ///< Returns the number of columns.


    /**
     * \brief Saves the AbstractMatrix to a file.
     */
    bool save(const std::string& filename) const;

    /**
     * \brief Loads the AbstractMatrix from a file.
     */
    virtual bool load(const std::string& filename)=0;

    virtual ~AbstractMatrix() {
        // Nothing to do here
    }

    virtual void print(std::ostream& out) const = 0; ///< Prints the matrix.

    /**
     * \brief Returns the value at the given row and column.
     */
    virtual double getCell(const size_t row, const size_t column=0) const = 0;

    /**
     * \brief Returns the string representation for a given type.
     */
    std::string strType() const;

protected:
    /**
     * \brief The matrix's type represents which nucleotides it considers.
     * \remarks To find which nucleotides we consider, convert the type to
     *  binary. Looking at the binary representation of a type_, relevant
     *  nucleotides occur wherever we see a "1" and nonrelevant ones wherever
     *  we see a "0". Note that we always consider the front nucleotide.
     *
     * EX: The type of 0 corresponds to considering only the front nucleotide,
     *     just like PATSER. (This is represented as "k", where "k" can be
     *     any base.)
     *
     * EX2: The type 1 corresponds to the dinucleotide case, where we consider
     *     two nucleotides at once. (This is represented as "kk", where each
     *     k can be any base).
     *
     * EX3: The type 5 is "101" in binary. Letting "m" represent nucleotides
     *     we don't care about, our results will be represented as "kmkk".
     *     IE, we consider the front nucleotide and the nucleotide to its
     *     immediate left, as well as the nucleotide three base pairs away.
     */
    size_t type_;

    /**
     * \brief Decomposes a base-10 type into a useful representation.
     * \return Each integer inside this vector corresponds to a relevant
     * nucleotide that many base pairs away from the front one. For instance,
     * if "3" were in the vector then the nucleotide 3 base pairs to the left
     * of the one in front would be relevant.
     *
     * The integers are guaranteed to be sorted least to greatest. Hence, at
     * the front of this vector will be "0" (as the front nucleotide is
     * always relevant), and so on.
     */
    std::vector<size_t> decomposeType() const;

    /**
     * \brief Generates an ID from a string of nucleotides.
     * \return An ID is a base-10 representation of a series of nucleotides
     * which are in base 4.
     * \remarks I have set A = 0, C = 1, G = 2, and T = 3. Hence the string
     *   "AAACT" would equal 7.
     */
    static size_t nucStringToID(const std::string& nucleotides);

    /**
     * \brief Generates a string of nucleotides from an ID.
     * \return The string in question is guaranteed to have "N"s in locations
     * where the Matrix's type specifies that the nucleotide there doesn't
     * matter.
     * \remarks For example, say our type is 2, corresponding to considering
     * the front nucleotide and the nucleotide two base pairs to its left.
     * This looks like "mkm". An ID of 6 would imply a string of "CG" were
     * there no gaps, but since there is a gap, we get "CNG" in this case.
     */
    std::string idToString(const size_t id) const;

    /**
     * \brief Generates a string of nucleotides from a base-10 representation.
     * \return This returns exactly the same thing as idToString, except
     * we don't consider gaps here at all. The length of the string outputted
     * is the size of how many columns are in the matrix (IE, our window
     * size).
     */
    std::string baseTenToNuc(const size_t id) const;

    /**
     * \return the row ID corresponding to the nucleotides at position
     * "location" within the string. This is all done according to the matrix's
     * type, so nucleotides we wouldn't consider are properly excluded.
     * Note that location represents the position of the last nucleotide of the
     * sequence.
     */
    size_t deref(const std::string& nucleotides, const size_t location) const;
};

#include "abstractmatrix-private.hpp"
// ABSTRACTMATRIX included
#endif
