/**
 * \file bgmatrix.cpp
 *
 * \authors Rowan Zellers
 *
 * \brief Implements the bgmatrix class.
 */

#include "bgmatrix.hpp"
#include <string>
#include <cstddef>
#include <iostream>
#include <fstream>
#include <vector>
#include <map>
#include <sstream>

using namespace std;

BgMatrix::BgMatrix(size_t type)
  : AbstractMatrix(type)
{
    // Nothing else needed.
}

BgMatrix::~BgMatrix()
{
    // Nothing else needed.
}

size_t BgMatrix::columns() const
{
    return 0;
}

double BgMatrix::getCell(const size_t row, const size_t column) const
{
    if (column != 0) {
        cerr << "Error: Column not equal to 0 for a BgMatrix." << endl;
        return data_[row];
    }

    if (row >= data_.size()) {
        cerr << "Error: row too large for getcell" << endl;
        return data_[0];
    }

    return data_[row];
}

bool BgMatrix::load(const std::string& filename)
{   
    // Make sure the file exists.
    ifstream input(filename.c_str());
    if (!input) {
        return false;
    }

    string s;
    input >> s;

    size_t i;
    // The first 6 characters in the loaded file are useless, so
    // strip them away then parse the ID into a size_t.
    stringstream convert(s.substr(6));
    if ( !(convert >> i) ) {
        return false;
    }
    type_ = i;

    // Get the specifications of the matrix and set everything to 0.
    assert(data_.empty());
    size_t numrows = rows();
    data_.resize(numrows);
    input >> s;

    // Push back whatever number occurs after the tab character, since
    // a BgMatrix will only have one column.
    size_t begin = s.find_first_of('\t');
    double pushee = atof((s.substr(begin)).c_str());
    data_.push_back(pushee);

    // Do the same thing as above for the remaining strings (kinda ugly).
    while(getline(input, s)) {
        size_t begin = s.find_first_of('\t');
        double pushee = atof((s.substr(begin)).c_str());
        data_.push_back(pushee);
    }

    assert(data_.size() == numrows);

    return true;
}
bool BgMatrix::loadf(const string& background)
{
    // Make sure the file exists.
   ifstream input(background.c_str());
    if (!input){
        return false;
    }

    // For background frequencies we just want 1 column. But, we will have
    // to iterate to the end of the string anyways, so we still have to find
    // l - (m+k) + 1.
    vector<size_t> positions = decomposeType();
    size_t startpos = *max_element(positions.begin(), positions.end());
    size_t numrows = rows();


    // Initialize the matrix by setting everything to 0
    assert(data_.empty());
    data_.resize(numrows);

    string genome("");
    string s;
    size_t count = 0;
    // If we hit a blank line then we add info from the chromosome in.
    while (input >> s) {
        // Count the number of occurences of a certain motif.
        // Note that 'positions' represents "m", the number of nucleotides
        // that we care about. We start at position m+k-1 (of the string!),
        // and end until we hit the end of string s.
        if (s.empty()) {
            size_t columns = genome.length() - startpos;

            for (size_t i = 0; i < columns; ++i) {
                // The relative substring starts at i and is [startpos] long.
                // If the string doesn't contain any N's then we can increment
                // the background matrix data.
                string relativestring = genome.substr(i, startpos+1);
                if (relativestring.find('N') == string::npos) {
                    ++( data_[deref(genome, i+startpos)] );
                    ++count;
                }
            }

            genome = "";
        } else {
            genome += s;
        }
    }

    // In case there's no blank line at the end, catch the extra chromosome.
    if (!genome.empty()) {
        size_t columns = genome.length() - startpos;

        for (size_t i = 0; i < columns; ++i) {
            // The relative substring starts at i and is [startpos] long.
            // If the string doesn't contain any N's then we can increment
            // the background matrix data.
            string relativestring = genome.substr(i, startpos+1);
            if (relativestring.find('N') == string::npos) {
                ++( data_[deref(genome, i+startpos)] );
                ++count;
            }
        }
        
        genome = "";
    }

    // Now divide everything by count and incorporate smoothing.
    for (size_t i = 0; i < numrows; ++i) {
        data_.at(i) += 1.0/( (double) numrows); // Add 1/4^mu to numerator,
        // and 1 to denominator.
        // Mu is the number of nucleotides we consider. It's also
        // just the number of rows.
        double denom = count + 1.0;
        (data_.at(i)) /= denom;
    }



    return true;
}


void BgMatrix::print(std::ostream& out) const
{
    out << "Printing background frequencies for type " << type_ << endl;
    for (size_t i = 0; i < data_.size(); ++i) {
        out << idToString(i) << ": ";
        if(data_[i] == 0) {
            out << "0";
        } else {
            out << data_[i];
        }
        out << endl;
    }
}


