/**
 * \file bgmatrix.hpp
 *
 * \authors Rowan Zellers
 *
 * \brief Declares the bgmatrix class.
 */

 #ifndef BGMATRIX_HPP_INCLUDED
 #define BGMATRIX_HPP_INCLUDED 1

#include "abstractmatrix.hpp"
#include <string>
#include <cstddef>
#include <iostream>
#include <fstream>
#include <vector>
#include <map>


/**
 * \class Bgmatrix
 * \brief A Bgmatrix is a class that contains background frequencies.
 *        Together with a FreqMatrix it can construct a WeightMatrix.
 * \details A background matrix has (in this implementation) "0"
 *     columns. Really though, its data structure is just a vector
 *     so it has 1 column. 
 */

 class BgMatrix : public AbstractMatrix {
public:

    BgMatrix(size_t type=0); ///< Constructs a matrix according to its type.

    /**
     * \brief From a filename containing the entire genome, fill in the
     * background matrix.
     * \code For each string in file:
     *           merge into one string, s.
     *   Iterating through s, at each iteration:
     *       Get the ID corresponding to the relevant nucleotides nearby.
     *       Increment the row of the matrix (corresponding to that ID) by 1.
     *   Divide each row of the matrix by the total number of
     *   nucleotide-groups we encountered.
     * \details The file in question should have multiple chromosomes,
     *      each separated by a blank line. (This is so that the region
     *      between chromosomes doesn't get scored.) 
     */
    bool loadf(const std::string& filename);


    /**
     * \brief Loads a BgMatrix from a file with another BgMatrix.
     */
    virtual bool load(const std::string& filename);

    virtual size_t columns() const; ///< Returns 0.

    virtual ~BgMatrix();

    virtual void print(std::ostream& out) const; ///< Prints the matrix.

    virtual double getCell(const size_t row, const size_t column=0) const;

private:

    std::vector<double> data_; ///< The background frequencies.

};
#endif
