/**
 * \file freqmatrix.cpp
 *
 * \authors Rowan Zellers
 *
 * \brief Implements the freqmatrix class.
 */

#include "freqmatrix.hpp"
#include <string>
#include <cstddef>
#include <iostream>
#include <fstream>
#include <vector>
#include <map>
#include <cmath>
#include <cassert>
#include <sstream>

using namespace std;

FreqMatrix::FreqMatrix(size_t type)
  : AbstractMatrix(type)
{
    // Nothing else needed.
}

FreqMatrix::~FreqMatrix()
{
    // Nothing else needed.
}

size_t FreqMatrix::columns() const
{
    return data_[0].size();
}

double FreqMatrix::getCell(const size_t row, const size_t column) const
{
    if (row >= data_.size()) {
        cerr << "Error: row too large for getcell, FreqMatrix" << endl;
        return 0;
    }

    if (column >= data_[row].size()) {
        cerr << "Error: column too large for getcell, freqmatrix" << endl;
        return 0;
    }

    return (data_[row])[column];
}

bool FreqMatrix::load(const std::string& filename)
{
    // Note: this works in exactly the same way as does WeightMatrix::load.
    ifstream input(filename.c_str());
    if (!input) {
        return false;
    }
    // Check if it's a background matrix.
    string s;
    getline(input, s);

    // The first 6 characters of the first line are reserved for text.
    // Convert the ID (located afterwards) into an integer, and update type_.
    size_t i;
    stringstream convert(s.substr(6));
    if ( !(convert >> i) ) {
        return false;
    }
    type_ = i;

    // Get the specifications of the matrix and set everything to 0.
    assert(data_.empty());
    size_t numrows = rows();
    data_.resize(numrows);
    getline(input, s);

    // For each string, start at position 0. Make a substring between two
    // adjacent tabs, and convert that substring to a double. Push back the
    // double onto our data_ vector. Continue to do this until we hit the last
    // tab.
    size_t startpos=0;
    size_t check = s.find_last_of('\t');
    while (startpos < check) {
        size_t begin = s.find_first_of('\t', startpos);
        size_t end = s.find_first_of('\t', begin+1);
        double pushee = atof((s.substr(begin, end)).c_str());
        startpos = end;
        data_[0].push_back(pushee);
    }

    // There's still one chunk of data after the last tab.
    double lastpushee = atof(s.substr(startpos).c_str());
    data_[0].push_back(lastpushee);

    // Do the same thing as above for the remaining strings (kinda ugly).
    size_t currentline = 1;
    while(getline(input, s)) {
        startpos = 0;
        check = s.find_last_of('\t');
        while (startpos < check) {
            size_t begin = s.find_first_of('\t', startpos);
            size_t end = s.find_first_of('\t', begin+1);
            double pushee = atof((s.substr(begin, end)).c_str());
            startpos = end;
            data_[currentline].push_back(pushee);
        }
        double lastpushee = atof(s.substr(startpos).c_str());
        data_[currentline].push_back(lastpushee);
        ++currentline;

    }

    return true;
}


bool FreqMatrix::loadf(const string& foreground)
{
    ifstream input(foreground.c_str());

    if (!input){
        return false;
    }

    // Get the specifications of the matrix by reading in one string.
    // With no gaps a matrix has l - m + 1 columns, where M is the
    // number of considered nucleotides and l is the length of the string.
    // Hence, PATSER has l columns. With gaps, we will have l - (m+k) + 1
    // columns, where k is the number of skipped nucleotides.
    // We can find m+k-1 by picking the maximal element from the vector of
    // relevant nucleotides given by DecomposeType.
    string s;
    input >> s;
    assert(s.length() > log2(type_) + 1 && "Your window is too large!");
    vector<size_t> positions = decomposeType();
    size_t startpos = *max_element(positions.begin(), positions.end());
    size_t columns = s.length() - startpos;
    size_t numrows = rows();

    // Initialize the matrix by setting everything to 0.
    assert(data_.empty());
    data_.resize(numrows);
    for (size_t i = 0; i < numrows; ++i) {
        data_[i].resize(columns);
    }

    // Count the number of occurences of a certain motif.
    // Note that 'positions' represents "m", the number of nucleotides
    // that we care about. We start at position m+k-1 (of the string!),
    // and end until we hit the end of string s.
    //
    // Dereference the pseudo iterator i with respect to string s.
    // The column id will be such that when i is at its beginning,
    // the column id = 0.
    for (size_t i = 0; i < columns; ++i) {
        ++( data_[deref(s, i+startpos)].at(i) );

    }

    l_ = 1; // L represents the number of separate strings that
                     // we consider. (We've already seen one string).

    while (input >> s) {
        for (size_t i = 0; i < columns; ++i) {
            ++( data_[deref(s, i+startpos)].at(i) );
        }
        ++l_;
    }


    // // Now divide everything by l_.
    // for (size_t i = 0; i < numrows; ++i) {
    //     for (size_t j = 0; j < columns; ++j) {
    //         (data_.at(i)).at(j) += 1; // Add 1, and then divide by 
    //         // l_ + 4^mu.
    //         // Mu is the number of nucleotides we consider. It's also
    //         // just the number of rows.
    //         double denom = l_ + numrows;
    //         (data_.at(i)).at(j) /= denom;
    //     }
    // }

    // update, we are storing just the raw n_{b,i} stuff!

    return true;
}

size_t FreqMatrix::getl() const
{
    cerr << "l_" << l_ << endl;
    return l_;
}


void FreqMatrix::print(std::ostream& out) const
{
    out << "Printing frequency matrix with type " << type_ << endl;
    for (size_t i = 0; i < data_.size(); ++i) {
        out << idToString(i) << ": ";
        for(size_t j = 0; j < data_[i].size(); ++j) {
            size_t currentwidth = out.width(VISIBLE_DIGITS+1);

            if(data_[i][j] == 0) {
                out << "0";
                for (size_t i = 0; i < VISIBLE_DIGITS-1; ++i) {
                    out << " ";
                }
            } else {
                stringstream ss;
                ss << data_[i][j];
                string s = ss.str();
                for (size_t i = 0; i < VISIBLE_DIGITS && i < s.length(); ++i) {
                    out << s.at(i);
                }
            }
            out.width(currentwidth);
        }
        out << endl;
    }
}


