/**
 * \file freqmatrix.hpp
 *
 * \authors Rowan Zellers
 *
 * \brief Declares the freqmatrix class.
 */

 #ifndef FREQMATRIX_HPP_INCLUDED
 #define FREQMATRIX_HPP_INCLUDED 1

#include "abstractmatrix.hpp"
#include <string>
#include <cstddef>
#include <iostream>
#include <fstream>
#include <vector>
#include <map>


/**
 * \class Freqmatrix
 * \brief A freqmatrix is a class that contains frequencies.
 * \details Each matrix has a type according to which nucleotides it considers.
 */

 class FreqMatrix : public AbstractMatrix {
public:

    FreqMatrix(size_t type=0); ///< Constructs a matrix according to its type.

    /**
     * \brief From a filename containing a bunch of (aligned!) strings of
     * ChIP/whatever data, makes a frequency matrix.
     * \code      For each string in data:
     *    At each position within the string, obtain the ID corresponding
     *       to the nearby relevant nucleotides.
     *    Increment the cell given by that ID (the row) and our position in
     *       the string (the column) by 1.
     * Divide each cell in the matrix by the number of strings that we saw.
     * \details Each aligned sequence should be on its own line. They
     *        should all have the same length (obviously). Also they have to
     *        be shorter than the length of the shortest ChIP peak. That's
     *        about it!
     */
    bool loadf(const std::string& filename);

    /**
     * \brief Loads a freqmatrix from a file with a saved FreqMatrix.
     */
    virtual bool load(const std::string& filename);

    virtual size_t columns() const;

    virtual ~FreqMatrix();

    virtual void print(std::ostream& out) const; ///< Prints the matrix.

    double getCell(const size_t row, const size_t column) const;

    size_t getl() const;

private:

    std::vector<std::vector<double> > data_; ///< The frequencies.

    size_t l_; ///< L, the number of items in the aligned sequences.
};
#endif
