/**
 * \file marz.cpp
 *
 * \authors Rowan Zellers
 *
 * \brief This file contains the "main driver" function for the marz program.
 */

#include <cassert>
#include <cctype>
#include <iostream>
#include "freqmatrix.cpp"
#include "bgmatrix.cpp"
#include "weightmatrix.cpp"
#include "peakloader.cpp"
#include "peakscorer.cpp"
#include "resultsmaker.cpp"
#include "resultsprinter.cpp"

#include <map>

using namespace std;

size_t NUM_TRIALS = 100; ///< Extra trials besides trying all of the cutoffs
                         ///< in the aligned sequences.

size_t NUM_SHUFFLES = 100; ///< Controls how many times we shuffle ChIP peaks
                          ///< to get "fake" data, for testing matrices.

bool SCOREGTE = true; ///< If true then the score can be >= than the cutoff
                      ///< for the matrix to get a point under my algorithm.

bool CORR_SCORE_NUCS = true; ///< If true then the correlation vector scores
                            ///< nucleotides, if false, then binding sites.

double EPSILON = 0.5; ///< Controls the range of uncertainty for RZ scores.



int main(int argc, const char* argv[])
{

    // Display a message if the wrong number of arguments are given.
    if (argc < 4) {
        cout << "Usage: Type ./marz [SEQUENCES.txt][GENOME.txt]" <<
                "[numfiles] [filepath] [Type ID] [Threshold] [Name of output]" << endl
        << "SEQUENCES.txt refers to aligned footprint/PBM data." << endl
        << "GENOME.txt refers to a text file with the entire genome." << endl
        << "numfiles is how many files are in the filepath directory." << endl
        << "(these files must be named 0 ... [numfiles - 1])" << endl
        << "filepath is the location of the ChIP data" << endl
        << "Type ID is the gapped n-mer matrix type to use" << endl
        << "Threshold is a number in the range [0,1] representing the" << endl
        << "threshold position to use." << endl
        << "Name of output is the base filename for the output data" << endl;
        return 0;
    }

    // Get the number of files in the directory.
    size_t numfiles = atoi(argv[3]);
    size_t matrixid = atoi(argv[5]);

    //Construct the matrix.
    FreqMatrix freqmatrix(matrixid);
    BgMatrix backgroundmatrix(matrixid);
    WeightMatrix weightmatrix(matrixid);
    if (!freqmatrix.loadf( string(argv[1]))) {
        cerr << "File " << string(argv[1]) << "invalid" << endl;
    }
    if (!backgroundmatrix.loadf( string(argv[2]))) {
        cerr << "File " << string(argv[2]) << "invalid" << endl;
    }

    // Initialize the weight matrix from the frequency and
    // background matrices, and compute thresholds.
    weightmatrix.computeWeights(freqmatrix, backgroundmatrix);
    if (!weightmatrix.loadCutoffs(string(argv[1]))) {
        cerr << "file " << string(argv[1]) << "invalid" << endl;
    }
    cout << "Frequency matrix for " << string(argv[1]) << endl;
    freqmatrix.print(cout);
    cout << "Background matrix for " << string(argv[1]) << endl;
    backgroundmatrix.print(cout);
    cout << "Weight matrix for " << string(argv[1]) << endl;
    weightmatrix.print(cout);

    // Load in the ChIP peaks, and randomize as needed.
    PeakLoader peakstrings(numfiles, NUM_SHUFFLES);
    for (size_t i = 0; i < numfiles; ++i) {

        // convert i to a string without using C++ 11.
        stringstream ss;
        ss << i;
        string filename = string(argv[4]);
        filename += "/";
        filename += ss.str();

        //Our ChIP peaks are saved 0, 1, ... numpeaks-1, so load them in:
        peakstrings.load(filename, i);
    }

//
//  Stage 2 of the algorithm: PEAKSCORER'S REVENGE.
//
    PeakScorer scoredpeaks(numfiles, NUM_SHUFFLES);
    for (size_t i = 0; i < NUM_SHUFFLES+1; ++i) {
        for (size_t j = 0; j < numfiles; ++j) {
            scoredpeaks.score(weightmatrix, peakstrings, j, i);
        }
    }

//
//  Stage 3 of the algorithm: ResultsMaker fun
//
    // Get the proper threshold.
    double threshold = weightmatrix.getPercentileCutoff(atof(argv[6]));
    ResultsMaker compiledresults(numfiles, NUM_SHUFFLES, SCOREGTE, CORR_SCORE_NUCS, EPSILON);
    compiledresults.testCutoff(scoredpeaks, threshold);

    string filename = string(argv[7]);
    ResultsPrinter(matrixid, compiledresults, filename);


    return 0;
}
