/**
 * \file peakloader.cpp
 *
 * \authors Rowan Zellers
 *
 * \brief Implements the peakloader class.
 */

#include "peakloader.hpp"
#include <string>
#include <cstddef>
#include <iostream>
#include <fstream>
#include <vector>
#include <map>
#include <cmath>
#include <cstdlib>
#include <ctime>

using namespace std;

PeakLoader::PeakLoader(const size_t numpeaks,  const size_t numscrambles)
  : numpeaks_(numpeaks), numscrambles_(numscrambles)
{
    legitnucs_.resize(numpeaks);
    scramblednucs_.resize(numpeaks);
    for (size_t i = 0; i < numpeaks; ++i) {
        scramblednucs_[i].resize(numscrambles);
    }
}


bool PeakLoader::load(const string& filename, const size_t peakID)
{
    // Check that the file exists.
    ifstream input(filename.c_str());
    if (!input){
        return false;
    }

    // Load in the file as one giant string.
    string s;
    while (input >> s) {
        legitnucs_[peakID] += s;
    }

    // If the ChIP peak is less than 10bp long something's up.
    if (legitnucs_[peakID].length() < 10) {
        cerr << "SOMETHING IS VERY WRONG WITH FILENAME " << filename << endl;
        cerr << legitnucs_[peakID];
        assert(false);
    }

    // Resize the vectors according to the length of the string.
    // Note that if we have the bindingsites_ vector represent where
    // the binding sites START then this vector will have to be resized
    // to the length of the ChIP peak, minus the length of the window as
    // binding sites can't start at the end.

    srand(time(0));
    for (size_t i = 0; i < numscrambles_; ++i) {
        if (!permuteNucleotides(peakID, i)) {
            return false;
        }
    }
    return true;

}

std::string& PeakLoader::getPeak(const size_t peakID, const size_t scrambleID)
{
    if (scrambleID == 0) {
        return legitnucs_[peakID];
    } else {
        return scramblednucs_[peakID][scrambleID - 1];
    }
}


bool PeakLoader::permuteNucleotides(const size_t peakID, const size_t scrambleID)
{
    assert(peakID < numpeaks_ && scrambleID < numscrambles_);


    scramblednucs_[peakID][scrambleID] = legitnucs_[peakID];
    random_shuffle(scramblednucs_[peakID][scrambleID].begin(),
                   scramblednucs_[peakID][scrambleID].end());

    return true;
}

size_t PeakLoader::randomInt(size_t maximum)
{
    // We take rather unnecessary care here to avoid skewing the
    // distribution.  Only really necessary if maximum is very
    // large.
    ++maximum;
    long divisor = RAND_MAX / maximum;
    size_t candidate;

    do {
        candidate = random() / divisor;
    } while (candidate >= maximum);

    return candidate;
}
