/**
 * \file peakloader.hpp
 *
 * \authors Rowan Zellers
 *
 * \brief Declares the PeakLoader class.
 */

 #ifndef PEAKLOADER_HPP_INCLUDED
 #define PEAKLOADER_HPP_INCLUDED 1

#include <string>
#include <cstddef>
#include <iostream>
#include <fstream>
#include <vector>
#include <map>

/**
 * \class PeakLoader
 * \brief A peakloader is a class that loads in a bunch of
 *        ChIP peaks. It then generates a bunch of sets of
 *        scrambled peaks for each real peak.
 */

class PeakLoader {
public:
    /**
     * \brief Constructs a peakloader that will hold numpeaks ChIP peaks,
     *        with each one having numscrambles scrambled representations.
     */
    PeakLoader(const size_t numpeaks,  const size_t numscrambles = 100);

    /**
     * \brief Loads in the file "filename", corresponding to the peak with
     *        ID peakID. PeakID can range from 0 to numpeaks_ - 1,
     *        aka the first peak has ID 0.
     */
    bool load(const std::string& filename, const size_t peakID);

    /**
     * \brief Returns a reference to a string corresponding to the peakIDth
     *        peak. If scrambleID = 0, this will be the real peak. If not,
     *        it will be the IDth scramble of that peak.
     * \warning the string in question SHOULD NOT BE MODIFIED!
     */   
    std::string& getPeak(const size_t peakID, const size_t scrambleID=0);

    static size_t randomInt(size_t maximum); ///< Returns a random int.

private:
    size_t numpeaks_;
    size_t numscrambles_;
    std::vector<std::string> legitnucs_; ///< String of ChIP data.
    std::vector<std::vector<std::string> > scramblednucs_; ///< Contains 
                                            ///< numscrambles_ strings
                                             ///< that are scrambled versions
                                             ///< of legitnucs_.

    /**
     * \brief Randomizes the string of legit nucleotides, creating
     * the scrambleIDth scramble of the peakIDth ChIP Peak.
     * Be sure scrambleID < numscrambles_ and peakID < numpeaks!
     */ 
   bool permuteNucleotides(const size_t peakID, const size_t scrambleID);

};
#endif
