/**
 * \file peakscorer.hpp
 *
 * \authors Rowan Zellers
 *
 * \brief Declares the peakscorer class.
 */

 #ifndef PEAKSCORER_HPP_INCLUDED
 #define PEAKSCORER_HPP_INCLUDED 1

#include <string>
#include <cstddef>
#include <iostream>
#include <fstream>
#include <vector>
#include <map>
#include "weightmatrix.hpp"
#include "peakloader.hpp"

/**
 * \class PeakScorer
 * \brief A PeakScorer object analyzes a peakloader object containing
 *        strings of all (real and scrambled) ChIP peaks. Then, according
 *        to a given matrix, it scores all of these strings, in a way that
 *        the ResultsMaker object can analyze them over various threshholds.
 */

 class PeakScorer {
public:
    /**
     * \brief Constructs a peakscorer that will hold numpeaks ChIP peaks,
     *        with each one having numscrambles scrambled representations.
     */
    PeakScorer(const size_t numpeaks,  const size_t numscrambles = 100);

    /**
     * \brief Scores the peakIDth peak with the given matrix. If scrambleID is
     *        0 it will be a real matrix. Otherwise it will be the ScrambleIDth
     *        fake matrix.
     */   
    void score(const WeightMatrix& matrix, PeakLoader& peaks, 
        const size_t peakID, const size_t scrambleID=0);

    /**
     * \brief Returns a reference to a vector with scores for the peakIDth
     *        peak. If scrambleID = 0, this will be the real peak. If not,
     *        it will be the scrambleIDth scramble of that peak.
     * \details Please don't modify the vector!!!
     */   
    std::vector<double>& getScores(const size_t peakID, 
                                   const size_t scrambleID=0);

    /**
     * \brief Returns the length of the scores for a given ChIP peak. Note
     *        the first ChIP peak has ID = 0.
     */   
    size_t getLength(const size_t peakID) const;

private:
    size_t numpeaks_;
    size_t numscrambles_;
    std::vector<std::vector<double>> legitscores_; ///< Contains real scores for all ChIP
                                      ///< peaks.
    std::vector<std::vector<std::vector<double>>> scrambledscores_; ///< Contains 
                                            ///< scores for all false ChIP
                                            ///< ChIP peaks.
};
#endif
