/**
 * \file resultsmaker.cpp
 *
 * \authors Rowan Zellers
 *
 * \brief Implements the resultsmaker class.
 */

#include "resultsmaker.hpp"
#include <string>
#include <cstddef>
#include <iostream>
#include <fstream>
#include <vector>
#include <map>
#include <cmath>
#include <cstdlib>
#include <ctime>
#include "results.hpp"

using namespace std;


ResultsMaker::ResultsMaker(const size_t numpeaks,  const size_t numscrambles,
    const bool scoregte, const bool corrscorenucs,
    const double epsilon)
  : numpeaks_(numpeaks), numscrambles_(numscrambles),
    scoregte_(scoregte), corrscorenucs_(corrscorenucs), epsilon_(epsilon)
{
}

void ResultsMaker::testCutoff(PeakScorer& peaks, const double threshold)
{
    // For each ChIP peak, the matrix could either get or not get a point.
    // For each real ChIP peak then the matrix could either get or not get
    // a false negative.
    // For each false ChIP peak then the matrix could either get or not get
    // a false positive.
    double points=0;
    size_t falsenegatives=0;
    size_t falsepositives=0;
    size_t hits = 0;
    size_t epsilons = 0;
    double urfalsepositives = 0;

    // For each peak, get and analyze the scores from the Peaks object.
    for (size_t i = 0; i < numpeaks_; ++i) {
        vector<double> scores = peaks.getScores(i);

        //Keep track of the number of real and (average) fake binding sites we
        // see on this run.
        double realSites = 0;
        double avgfakeSites = 0;

        // Go through the scores on the ChIP peak. If the peak passes
        // the threshold, then we update the binding vector corresponding
        // to the real binding sites that we see, and return 1. Else, we return
        // 0. If we saw no real peaks, then we have a false negative.
        for (size_t j = 0; j < scores.size(); ++j ) {
            realSites += testReal(scores[j], threshold);
        }
        if (realSites == 0) {
            ++falsenegatives;
        }


        double currentfalsepositives=0;
        double currentfakeSites = 0;
        // Go through the scores on the fake ChIP peaks.
        for (size_t j = 0; j < numscrambles_; ++j) {
            vector<double> fakescores = peaks.getScores(i,j+1);
            size_t fakeSites = 0;
            for (vector<double>::iterator k = fakescores.begin();
                k != fakescores.end(); ++k) {
                fakeSites += testFake(*k, threshold);
            }

            //If we found any fake sites on this scrambled ChIP peak,
            // increment the average fake positives accordingly.
            if (fakeSites != 0) {
                currentfalsepositives += 1;
            }

            currentfakeSites += fakeSites;

        }
        currentfalsepositives = currentfalsepositives/( (double) numscrambles_);
        avgfakeSites += currentfakeSites / ((double) numscrambles_);
        falsepositives += size_t(currentfalsepositives + epsilon_);
        urfalsepositives += currentfalsepositives;

        // As detailed in the tex file, if R > F + E, we get a point.
        // If F + E >= R >= F - E we get half a point.
        // If F - E > R then we get 0 points.
        if (realSites > avgfakeSites + epsilon_) {
            ++points;
            ++hits;
        } else if (avgfakeSites + epsilon_ >= realSites &&
                                 realSites >= avgfakeSites - epsilon_) {
            points += 0.5;
            ++epsilons;
        }
    }

    // Update the results...
    testresults_.hits_ = hits;
    testresults_.epsilons_ = epsilons;
    testresults_.score_ = (points)/((double) numpeaks_);
    testresults_.unroundedfalsepositives_ = urfalsepositives;    
    testresults_.falsepositives_ = falsepositives;
    testresults_.falsenegatives_ = falsenegatives;
    testresults_.threshold_ = threshold;
}

Results& ResultsMaker::fetchResults()
{
    return testresults_;
}

size_t ResultsMaker::testReal(const double& score, const double& threshold)
{
    bool isbindingsite = (score > threshold ||
                         (scoregte_ && score >= threshold));

    if (!isbindingsite) {
        return 0;
    }


    return 1;
}

size_t ResultsMaker::testFake(const double& score, const double& threshold) const
{
    bool isbindingsite = (score > threshold || (scoregte_ && score >= threshold));
    if (isbindingsite) {
        return 1;
    }
    return 0;
}


