/**
 * \file resultsmaker.hpp
 *
 * \authors Rowan Zellers
 *
 * \brief Declares the resultsmaker class.
 */

 #ifndef RESULTSMAKER_HPP_INCLUDED
 #define RESULTSMAKER_HPP_INCLUDED 1

#include <string>
#include <cstddef>
#include <iostream>
#include <fstream>
#include <vector>
#include <map>
#include <utility>
#include "results.hpp"
#include "peakscorer.hpp"
#include "weightmatrix.hpp"

/**
 * \class ResultsMaker
 * \brief A ResultsMaker object analyzes a file containing scored ChIP
 *        data (both real and fake) for a given matrix. It then tests
 *        how many true positives and false positives we see, for a given
 *        threshhold along that data.
 * \details Documentation as to the function of the datamembers
 *          is contained within Marz.cpp.
 */

 class ResultsMaker {
public:
    friend class PearsonCorrelator;
    /**
     * \brief Constructs a resultsmaker that will hold numpeaks ChIP peaks,
     *        with each one having numscrambles scrambled representations.
     */
    ResultsMaker(const size_t numpeaks,  const size_t numscrambles = 100,
                 const bool scoregte = true,
                 const bool corrscorenucs = true, const double epsilon = 0);

     /**
     * \brief Tests the given threshhold and matrix type over all ChIP peaks,
     *        real and fake. Saves the results to the ResultsMaker object.
     * \details Based on the silly design of PeakScorer, I can't call
     *          it by const reference, though that would be nice.
     */   
    void testCutoff(PeakScorer& peaks, const double cutoff);

     /**
     * \brief Returns a reference to the scores of the ResultsMaker's matrix
     *        for the trial testID.
     * \details If I could I'd have it return a constant reference. But
     *          unfortunately that's not possible, so you just have to
     *          promise NOT TO MODIFY THE RESULTS VECTOR EVER. 
     */   
    Results& fetchResults();


private:
    size_t numpeaks_; 
    size_t numscrambles_; 
    bool scoregte_; 
    bool corrscorenucs_; 
    double epsilon_;
    Results testresults_; ///< The results from the test.

     /**
     * \brief A helper function that tests a portion of the real scores
     *        against the given threshhold. (It also saves data
     *        to the bindingVector regarding where we find binding
     *        nucleotides / sequences.)
     * \returns 1 if there's a binding site, 0 if there isn't one.
     */   
    size_t testReal(const double& score, 
        const double& threshold);

     /**
     * \brief A helper function that tests the a portion of fake scores
     *        against the given threshhold.
     * \returns 1 if there's a binding site, 0 if there isn't one.
     */  
    size_t testFake(const double& score, 
        const double& threshold) const;


};
#endif
