/**
 * \file weightmatrix.cpp
 *
 * \authors Rowan Zellers
 *
 * \brief Implements the weightmatrix class.
 */

#include "weightmatrix.hpp"
#include <string>
#include <cstddef>
#include <iostream>
#include <fstream>
#include <vector>
#include <map>
#include <cmath>
#include <sstream>

using namespace std;

WeightMatrix::WeightMatrix(size_t type)
  : AbstractMatrix(type)
{
      // nothing else needed
}

WeightMatrix::Sequence::Sequence(const size_t position, const string& nucs)
  : position_(position), nucs_(nucs)
{
    // nothing else needed
}

bool WeightMatrix::load(const std::string& filename)
{
    // Note: this works in exactly the same way as does FreqMatrix::load.
    ifstream input(filename.c_str());
    if (!input) {
        return false;
    }
    // Check if it's a background matrix.
    string s;
    getline(input, s);

    // The first 6 characters of the first line are reserved for text.
    // Convert the ID (located afterwards) into an integer, and update type_.
    size_t i;
    stringstream convert(s.substr(6));
    if ( !(convert >> i) ) {
        return false;
    }
    type_ = i;

    // Get the specifications of the matrix and set everything to 0.
    assert(data_.empty());
    size_t numrows = rows();
    data_.resize(numrows);
    getline(input, s);

    // For each string, start at position 0. Make a substring between two
    // adjacent tabs, and convert that substring to a double. Push back the
    // double onto our data_ vector. Continue to do this until we hit the last
    // tab.
    size_t startpos=0;
    size_t check = s.find_last_of('\t');
    while (startpos < check) {
        size_t begin = s.find_first_of('\t', startpos);
        size_t end = s.find_first_of('\t', begin+1);
        double pushee = atof((s.substr(begin, end)).c_str());
        startpos = end;
        data_[0].push_back(pushee);
    }

    // There's still one chunk of data after the last tab.
    double lastpushee = atof(s.substr(startpos).c_str());
    data_[0].push_back(lastpushee);

    // Do the same thing as above for the remaining strings (kinda ugly).
    size_t currentline = 1;
    while(getline(input, s)) {
        startpos = 0;
        check = s.find_last_of('\t');
        while (startpos < check) {
            size_t begin = s.find_first_of('\t', startpos);
            size_t end = s.find_first_of('\t', begin+1);
            double pushee = atof((s.substr(begin, end)).c_str());
            startpos = end;
            data_[currentline].push_back(pushee);
        }
        double lastpushee = atof(s.substr(startpos).c_str());
        data_[currentline].push_back(lastpushee);
        ++currentline;

    }

    return true;
}

bool WeightMatrix::loadCutoffs(const string& filename)
{

    // Fetch input
    ifstream input(filename.c_str());
    if (!input || !cutoffs_.empty()){
        return false;
    }

    // Score each sequence in the file, then add it to a vector of cutoffs.
    string s;
    vector<double> cutoffs;
    while (input >> s) {
        cutoffs_.push_back(scoreSequence(s));
    }

    // Sort the resulting vector of cutoffs, then find the location
    // corresponding to the proper percentile.
    // The 0th percentile cutoff will be the first in the sorted
    // cutoffs vector as it is sorted in ascending order.
    sort(cutoffs_.begin(), cutoffs_.end());
    return true;
}


double WeightMatrix::getPercentileCutoff(const double percentile) const
{
    if (percentile > 1 || percentile < 0) {
        cerr << "The percentile " << percentile << " is wrong." << endl;
        return 0;
    }
    size_t loc = (cutoffs_.size() - 1) * percentile;
    return cutoffs_[loc];
}

double WeightMatrix::getSpecificCutoff(const size_t loc) const
{
    assert(loc < cutoffs_.size());
    return cutoffs_[loc];
}

size_t WeightMatrix::getCutoffsLength() const
{
    return cutoffs_.size();
}



double WeightMatrix::getCell(const size_t row, const size_t column) const
{
    if (row >= data_.size()) {
        cerr << "Error: row too large for getcell, WeightMatrix" << endl;
        return 0;
    }

    if (column >= data_[row].size()) {
        cerr << "Error: column too large for getcell, weightmatrix" << endl;
    }

    return (data_[row])[column];
}

bool WeightMatrix::computeWeights(const FreqMatrix& freqmatrix, const BgMatrix& bgmatrix)
{

    size_t columns = freqmatrix.columns();
    size_t numrows = rows();
    double L = freqmatrix.getl();

    // Initialize the matrix by setting everything to 0.
    data_.resize(numrows);
    for (size_t i = 0; i < numrows; ++i) {
        data_[i].resize(columns);
    }

    // Apply the weight function to everything in the row.
    for (size_t i = 0; i < numrows; ++i) {

        double e = bgmatrix.getCell(i); // Expected frequency. It's already been smoothed.

        for (size_t j = 0; j < columns; ++j) {

            // The latest formula

            double num = freqmatrix.getCell(i, j) + e;
            double denom = e * (L + 1.0);

            data_.at(i).at(j) = log(num/denom);

            // double n = 0;     // n = \sum_{b=1}^4 n_b
            // for (size_t k = 0; k < numrows; ++k) {
            //     n += freqmatrix.getCell(k, j);
            // }

            // // If the first term under the logarithm is greater than
            // // .01 * n / (numrows * e), then s = 0. Else,
            // // s = .01 * n / (numrows * e).
            // double s = SMOOTHING_CONSTANT*n/(numrows*e);
            // if (freqmatrix.getCell(i, j)/e > s) {
            //     s = 0;
            // }

            // smoothing will now be done inside freqmatrix.

        }
    }

    // Normalize the columns so that the max in each one is 0.
    for (size_t i = 0; i < columns; ++i) {
        double max_element = data_.at(0).at(i);
        for (size_t j = 0; j < numrows; ++j) {
            if (data_.at(j).at(i) > max_element) {
                max_element = data_.at(j).at(i);
            }
        }
        // Subtract everything by the max element.
        for (size_t j = 0; j < numrows; ++j) {
            data_.at(j).at(i) -= max_element;
        }
    }
    return true;
}

WeightMatrix::~WeightMatrix()
{
    // Nothing needed.
}


size_t WeightMatrix::getWindowLength() const
{
    vector<size_t> positions = decomposeType();
    size_t startpos = *max_element(positions.begin(), positions.end());
    return columns() + startpos;
}


double WeightMatrix::scoreSequence(const string& nucs) const
{
    vector<size_t> positions = decomposeType();
    size_t startpos = *max_element(positions.begin(), positions.end());

    // The size of the window will be the width of the matrix.
    size_t numcolumns = data_[0].size();


    // To score a sequence, we have to find the actual weight of the
    // sequence and the minimum possible weight of any sequence of
    // nucleotides, and we divide the first by the second.
    // We started at position startpos in the string of nucleotides nucs,
    // meaning when we derefenence the nucleotides to find the correct
    // address, we start dereferencing at startpos.
    double stringweight = 0;
    double minweightsum = 0;
    for (size_t i = 0; i < numcolumns; ++i) {
        stringweight += data_.at(deref(nucs, i+startpos)).at(i);
        minweightsum += minWeight(i);
    }

    // The maximum nucleotides are 0, so it seems suspicious if minweightsum
    // ever gives us 0!
    if (minweightsum == 0) {
        cerr << "Minweightsum shouldn't be returning 0" << endl;
        cerr << "The nucleotides that gave the error:" << nucs << endl;
        cerr << " The type: "<< type_;
        return 0;
    }

    double score = 1 - stringweight/minweightsum;

    return score;
}

void WeightMatrix::slideWindow(const std::string& nucs,
    std::vector<double>& scores, const size_t peakID) const
{
    // If the sequence's length is smaller than the window length,
    // something has gone wrong as this makes it impossible to score.
    size_t wlength = getWindowLength();
    size_t nlength = nucs.length();
    if (nlength < wlength) {
        cerr << "wlength is " << wlength << endl;
        cerr << "nlength " << nlength << endl;
        cerr << "peakid " << peakID << endl;
        assert("Window too large for the given sequence." &&
                                nlength >= wlength);
    }

    // Slide the window along the string of nucleotides "nucs".
    // Since we generate a substring of wlength nucleotides, the maximum
    // valid position we can be in is wlength nucleotides before the end.
    // Score each nucs and add it to the vector of results.
    scores.resize(nlength - wlength+1);
    for (size_t i = 0; i < nlength - wlength+1; ++i) {
        string relevantnucs = nucs.substr(i, wlength);
        double score = scoreSequence(relevantnucs);
        scores[i] = score;
    }
}

size_t WeightMatrix::columns() const
{
    return data_[0].size();
}

double WeightMatrix::minWeight(const size_t column) const
{
    if (data_[0].size() <= column) {
        cerr << "Error: column parameter too large for minWeight" << endl;
        return 0;
    }
    double minWeight = 0;
    size_t numrows = rows();
    for (size_t i = 0; i < numrows; ++i) {
        if (minWeight > data_.at(i).at(column)) {
            minWeight = data_.at(i).at(column);
        }
    }
    return minWeight;

}


void WeightMatrix::print(std::ostream& out) const
{
    out << "Printing weight matrix with type " << type_ << endl;
    for (size_t i = 0; i < data_.size(); ++i) {
        out << idToString(i) << ": ";
        for(size_t j = 0; j < data_[i].size(); ++j) {
            size_t currentwidth = out.width(VISIBLE_DIGITS+1);

            if(data_[i][j] == 0) {
                out << "0";
                for (size_t i = 0; i < VISIBLE_DIGITS-1; ++i) {
                    out << " ";
                }
            } else {
                stringstream ss;
                ss << data_[i][j];
                string s = ss.str();
                for (size_t i = 0; i < VISIBLE_DIGITS && i < s.length(); ++i) {
                    out << s.at(i);
                }
            }
            out.width(currentwidth);
        }
        out << endl;
    }
}


