/**
 * \file weightmatrix.hpp
 *
 * \authors Rowan Zellers
 *
 * \brief Declares the weightmatrix class.
 */

 #ifndef WEIGHTMATRIX_HPP_INCLUDED
 #define WEIGHTMATRIX_HPP_INCLUDED 1

#include "abstractmatrix.hpp"
#include "freqmatrix.hpp"
#include "bgmatrix.hpp"
#include <string>
#include <cstddef>
#include <iostream>
#include <fstream>
#include <vector>
#include <map>


/**
 * \class Weightmatrix
 * \brief A weightmatrix is a class that contains weights.
 * \details Each matrix has a type according to which nucleotides it considers.
 */
 class WeightMatrix : public AbstractMatrix {
public:
    /**
     * \brief Constructs a weight matrix according to its type.
     */
    WeightMatrix(size_t type=0);

    /**
     * \brief From a frequency matrix and a background matrix, computes
     *        a weight matrix.
     * \code  Construct a matrix with the same dimensions as the freqmatrix.
     * Sets each cell to ln(\frac{n_{bi}}{e_{bi}}+s_i), where:
     * n_{bi} is the cell in the frequency matrix with the same row and column.
     * e_{bi} is the cell in the background matrix with the same row.
     * n is the sum of the frequencies in column i.
     * s_i =  0 if \frac{n_{bi}}{e_{bi}} > SMOOTHING_CONSTANT n /(4e_{bi}).
     * Otherwise, s_i = SMOOTHING_CONSTANT*n/(4*e_{bi}) otherwise,
     *
     * For each column:
     *     Add a constant c to each cell such that the max weight in the column
     *     is 0.
     */
    bool computeWeights(const FreqMatrix& freqmatrix,
                                const BgMatrix& bgmatrix);

    /**
     * \brief Loads a weight matrix from a file containing a saved matrix.
     */
    virtual bool load(const std::string& filename);

    virtual size_t columns() const;

    virtual ~WeightMatrix();

    virtual void print(std::ostream& out) const; ///< Prints the matrix.

    double getCell(const size_t row, const size_t column) const;

    /**
     * \brief From a filename (containing aligned sequences), scores all of
     *        them, initializing the cutoffs_ vector.
     */
    bool loadCutoffs(const std::string& filename); ///< loads cutoffs.
    /**
     * \brief Returns the nth percentile cutoff.
     * \details Percentile must be in the range [0-1]. A percentile of 0
     *        means all of the aligned sequences count as binding sites
     *        IE it returns the lowest score out of all of the files.
     *        A percentile of .5 means half of the aligned sequences
     *        count as binding sites, and it returns the 50th percentile
     *        (approx.) score.
     */
    double getPercentileCutoff(const double percentile) const;


    /**
     * \brief Returns the nth cutoff in the cutoffs vector. The 0th
     *        cutoff will be the lowest one.
     * \details loc must be < than the length of the cutoffs vector!
     */
    double getSpecificCutoff(const size_t loc) const;

    size_t getCutoffsLength() const; ///< Returns the length of cutoffs_.

    /**
     * \brief A sequence is a substring of nucleotides located at a given
     * position.
     */
    struct Sequence {
        size_t position_;
        std::string nucs_;
        Sequence(const size_t position, const std::string& nucs);
    };

    /**
     * \brief Scores a string using the sliding window technique. The vector
     *        will be modified with the score at each position.
     * \code  Read in the file given by filename, fetching a string.
     * For each position in the string:
     *     Score what the window sees at that position.
     *     Add that score to the table, along with the nucleotide sequence.
     * Return the table.
     */
    void slideWindow(const std::string& nucs, std::vector<double>& scores,
        const size_t peakID = 9001) const;

    size_t getWindowLength() const;

private:

    std::vector<std::vector<double> > data_; ///< Stores the weights.

    std::vector<double> cutoffs_; ///< Stores cutoffs for everything in the
                                  ///< aligned sequences.

    /**
     * \brief Helper function that scores a string that is entirely the
     *    window length.
     */
    double scoreSequence(const std::string& nucs) const;

    /**
     * \brief Helper function that finds the minimum weight in a column.
     */
    double minWeight(const size_t column) const;

};
#endif
